@@ -62,6 +62,9 @@ _GL_POINTS = 0
 _GL_TRIANGLES = 4
 _GL_STRIP = 5
 
+_EYE = np.eye(4)
+_EYE.flags.writeable = False
+
 # specify dtypes with forced little endian
 float32 = np.dtype("<f4")
 uint32 = np.dtype("<u4")
@@ -280,9 +283,23 @@ def load_gltf(file_obj=None,
         # old versions of python/json need strings
         tree = json.loads(util.decode_text(data))
 
+    # gltf 1.0 is a totally different format
+    # that wasn't widely deployed before they fixed it
+    version = tree.get('asset', {}).get('version', '2.0')
+    if isinstance(version, str):
+        # parse semver like '1.0.1' into just a major integer
+        major = int(version.split('.', 1)[0])
+    else:
+        major = int(float(version))
+
+    if major < 2:
+        raise NotImplementedError(
+            'only GLTF 2 is supported not `{}`'.format(
+                version))
+
     # use the URI and resolver to get data from file names
     buffers = [_uri_to_bytes(uri=b['uri'], resolver=resolver)
-               for b in tree['buffers']]
+               for b in tree.get('buffers', [])]
 
     # turn the layout header and data into kwargs
     # that can be used to instantiate a trimesh.Scene object
@@ -313,7 +330,8 @@ def load_glb(file_obj,
     resolver : trimesh.visual.Resolver
       Object which can be used to load other files by name
     merge_primitives : bool
-      If True, each GLTF 'mesh' will correspond to a single Trimesh object
+      If True, each GLTF 'mesh' will correspond to a
+      single Trimesh object.
 
     Returns
     ------------
@@ -328,10 +346,15 @@ def load_glb(file_obj,
     head_data = file_obj.read(20)
     head = np.frombuffer(head_data, dtype="<u4")
 
-    # check to make sure first index is gltf
-    # and second is 2, for GLTF 2.0
-    if head[0] != _magic["gltf"] or head[1] != 2:
-        raise ValueError("file is not GLTF 2.0")
+    # check to make sure first index is gltf magic header
+    if head[0] != _magic["gltf"]:
+        raise ValueError('incorrect header on GLB file')
+
+    # and second value is version: should be 2 for GLTF 2.0
+    if head[1] != 2:
+        raise NotImplementedError(
+            'only GLTF 2 is supported not `{}`'.format(
+                head[1]))
 
     # overall file length
     # first chunk length
@@ -1135,23 +1158,7 @@ def specular_to_pbr(
     return result
 
 
-def _parse_materials(header, views, resolver=None):
-    """
-    Convert materials and images stored in a GLTF header
-    and buffer views to PBRMaterial objects.
-
-    Parameters
-    ------------
-    header : dict
-      Contains layout of file
-    views : (n,) bytes
-      Raw data
-
-    Returns
-    ------------
-    materials : list
-      List of trimesh.visual.texture.Material objects
-    """
+def _parse_textures(header, views, resolver=None):
     try:
         import PIL.Image
     except ImportError:
@@ -1182,6 +1189,27 @@ def _parse_materials(header, views, resolver=None):
                 images[i] = PIL.Image.open(util.wrap_as_stream(blob))
             except BaseException:
                 log.error("failed to load image!", exc_info=True)
+    return images
+
+
+def _parse_materials(header, views, resolver=None):
+    """
+    Convert materials and images stored in a GLTF header
+    and buffer views to PBRMaterial objects.
+
+    Parameters
+    ------------
+    header : dict
+      Contains layout of file
+    views : (n,) bytes
+      Raw data
+
+    Returns
+    ------------
+    materials : list
+      List of trimesh.visual.texture.Material objects
+    """
+    images = _parse_textures(header, views, resolver)
 
     # store materials which reference images
     materials = []
@@ -1206,9 +1234,13 @@ def _parse_materials(header, views, resolver=None):
                     pbr[k] = v
                 elif "index" in v:
                     # get the index of image for texture
-                    idx = header["textures"][v["index"]]["source"]
-                    # store the actual image as the value
-                    pbr[k] = images[idx]
+                    try:
+                        idx = header["textures"][v["index"]]["source"]
+                        # store the actual image as the value
+                        pbr[k] = images[idx]
+                    except BaseException:
+                        log.debug('unable to store texture',
+                                  exc_info=True)
             # create a PBR material object for the GLTF material
             materials.append(visual.material.PBRMaterial(**pbr))
 
@@ -1344,9 +1376,9 @@ def _read_buffers(header,
                 kwargs = {"metadata": {}, "process": False}
                 kwargs.update(mesh_kwargs)
                 kwargs["metadata"].update(metadata)
-
                 # i.e. GL_LINES, GL_TRIANGLES, etc
-                mode = p.get('mode')
+                # specification says the default mode is GL_TRIANGLES
+                mode = p.get('mode', _GL_TRIANGLES)
                 # colors, normals, etc
                 attr = p['attributes']
                 # create a unique mesh name per- primitive
@@ -1363,13 +1395,8 @@ def _read_buffers(header,
                         points=np.arange(len(kwargs['vertices'])))]
                 elif mode == _GL_POINTS:
                     kwargs["vertices"] = access[attr["POSITION"]]
-                elif mode is None or mode in (_GL_TRIANGLES, _GL_STRIP):
-                    if mode is None:
-                        # some people skip mode since GL_TRIANGLES
-                        # is apparently the de-facto default
-                        log.debug(
-                            'primitive has no mode! trying GL_TRIANGLES?')
-                        # get vertices from accessors
+                elif mode in (_GL_TRIANGLES, _GL_STRIP):
+                    # get vertices from accessors
                     kwargs["vertices"] = access[attr["POSITION"]]
                     # get faces from accessors
                     if 'indices' in p:
@@ -1383,8 +1410,9 @@ def _read_buffers(header,
                         # indices are apparently optional and we are supposed to
                         # do the same thing as webGL drawArrays?
                         kwargs['faces'] = np.arange(
-                            len(kwargs['vertices']),
+                            len(kwargs['vertices']) * 3,
                             dtype=np.int64).reshape((-1, 3))
+
                     if 'NORMAL' in attr:
                         # vertex normals are specified
                         kwargs['vertex_normals'] = access[attr['NORMAL']]
@@ -1501,7 +1529,7 @@ def _read_buffers(header,
         [meshes.pop(p, None) for p in mesh_pop]
 
     # make it easier to reference nodes
-    nodes = header["nodes"]
+    nodes = header.get("nodes", [])
     # nodes are referenced by index
     # save their string names if they have one
     # we have to accumulate in a for loop opposed
@@ -1533,23 +1561,35 @@ def _read_buffers(header,
         # otherwise just use the first index
         scene_index = 0
 
-    # start the traversal from the base frame to the roots
-    for root in header["scenes"][scene_index]["nodes"]:
-        # add transform from base frame to these root nodes
-        queue.append([base_frame, root])
+    if 'scenes' in header:
+        # start the traversal from the base frame to the roots
+        for root in header["scenes"][scene_index].get("nodes", []):
+            # add transform from base frame to these root nodes
+            queue.append((base_frame, root))
+
+    # make sure we don't process an edge multiple times
+    consumed = set()
 
     # go through the nodes tree to populate
     # kwargs for scene graph loader
     while len(queue) > 0:
         # (int, int) pair of node indexes
-        a, b = queue.pop()
+        edge = queue.pop()
+
+        # avoid looping forever if someone specified
+        # recursive nodes
+        if edge in consumed:
+            continue
+
+        consumed.add(edge)
+        a, b = edge
 
         # dict of child node
         # parent = nodes[a]
         child = nodes[b]
         # add edges of children to be processed
         if "children" in child:
-            queue.extend([[b, i] for i in child["children"]])
+            queue.extend([(b, i) for i in child["children"]])
 
         # kwargs to be passed to scene.graph.update
         kwargs = {"frame_from": names[a], "frame_to": names[b]}
@@ -1559,10 +1599,11 @@ def _read_buffers(header,
         # for the transform from parent to child
         if "matrix" in child:
             kwargs["matrix"] = np.array(
-                child["matrix"], dtype=np.float64).reshape((4, 4)).T
+                child["matrix"],
+                dtype=np.float64).reshape((4, 4)).T
         else:
             # if no matrix set identity
-            kwargs["matrix"] = np.eye(4)
+            kwargs["matrix"] = _EYE
 
         # Now apply keyword translations
         # GLTF applies these in order: T * R * S
@@ -1598,7 +1639,7 @@ def _read_buffers(header,
                     # save the name of the geometry
                     kwargs["geometry"] = geom_name
                     # no transformations
-                    kwargs["matrix"] = np.eye(4)
+                    kwargs["matrix"] = _EYE
                     kwargs['frame_from'] = names[b]
                     # if we have more than one primitive assign a new UUID
                     # frame name for the primitives after the first one
